@@ -1,8 +1,16 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+#
+# Copyright (C) 2020, Matthias Erll
+# Copyright (C) 2020, James R. Barlow (https://github.com/jbarlow83/)
+
+
 from enum import Enum
 from itertools import chain
 from typing import Iterable, List, Optional, Set, Tuple, Union, cast
 
-from pikepdf import Array, Dictionary, Name, Object, Pdf
+from pikepdf import Array, Dictionary, Name, Object, Page, Pdf, String
 
 
 class PageLocation(Enum):
@@ -28,21 +36,48 @@ ALL_PAGE_LOCATION_KWARGS = set(chain.from_iterable(PAGE_LOCATION_ARGS.values()))
 
 
 def make_page_destination(
-    pdf,
+    pdf: Pdf,
     page_num: int,
     page_location: Optional[Union[PageLocation, str]] = None,
-    **kwargs,
+    *,
+    left: Optional[float] = None,
+    top: Optional[float] = None,
+    right: Optional[float] = None,
+    bottom: Optional[float] = None,
+    zoom: Optional[float] = None,
 ) -> Array:
     """
     Creates a destination ``Array`` with reference to a Pdf document's page number.
 
     Arguments:
-        pdf: ``Pdf`` document object.
+        pdf: PDF document object.
         page_num: Page number (zero-based).
-        page_location: Optional page location, as a string or ``PageLocation`` enum.
-        kwargs: Optional keyword arguments for the page location, e.g. ``top``.
+        page_location: Optional page location, as a string or :enum:`PageLocation`.
+        left, top, right, bottom, zoom: Optional keyword arguments for specifying
+            a position on the page, used in conjunction with the page fit style
+            specified by *page_location*.
     """
-    res = [pdf.pages[page_num]]
+    return _make_page_destination(
+        pdf,
+        page_num,
+        page_location=page_location,
+        left=left,
+        top=top,
+        right=right,
+        bottom=bottom,
+        zoom=zoom,
+    )
+
+
+def _make_page_destination(
+    pdf: Pdf,
+    page_num: int,
+    page_location: Optional[Union[PageLocation, str]] = None,
+    **kwargs,
+) -> Array:
+    kwargs = {k: v for k, v in kwargs.items() if v is not None}
+
+    res: List[Union[Dictionary, Name]] = [pdf.pages[page_num].obj]
     if page_location:
         if isinstance(page_location, PageLocation):
             loc_key = page_location
@@ -53,9 +88,9 @@ def make_page_destination(
                 loc_key = PageLocation[loc_str]
             except KeyError:
                 raise ValueError(
-                    "Invalid or unsupported page location type {0}".format(loc_str)
-                )
-        res.append(Name('/{0}'.format(loc_str)))
+                    f"Invalid or unsupported page location type {loc_str}"
+                ) from None
+        res.append(Name(f'/{loc_str}'))
         dest_arg_names = PAGE_LOCATION_ARGS.get(loc_key)
         if dest_arg_names:
             res.extend(kwargs.get(k, 0) for k in dest_arg_names)
@@ -88,34 +123,42 @@ class OutlineItem:
             May be ``None`` for creating a new object. If present, an existing
             object is modified in-place during writing and original attributes
             are retained.
-        kwargs: Additional keyword arguments. Any of ``left``, ``top``,
-            ``bottom``, ``right``, or ``zoom``, they will be processed for
-            usage of extended page location types, e.g. /XYZ.
+        left, top, bottom, right, zoom: Describes the viewport position associated
+            with a destination.
 
     This object does not contain any information about higher-level or
     neighboring elements.
+
+    Valid destination arrays:
+        [page /XYZ left top zoom]
+        generally
+        [page, PageLocationEntry, 0 to 4 ints]
     """
 
     def __init__(
         self,
         title: str,
-        destination: Optional[Tuple[int, str, Object]] = None,
+        destination: Optional[Union[Array, String]] = None,
         page_location: Optional[Union[PageLocation, str]] = None,
         action: Optional[Dictionary] = None,
         obj: Optional[Dictionary] = None,
-        **kwargs,
+        *,
+        left: Optional[float] = None,
+        top: Optional[float] = None,
+        right: Optional[float] = None,
+        bottom: Optional[float] = None,
+        zoom: Optional[float] = None,
     ):
         self.title = title
         self.destination = destination
         self.page_location = page_location
         self.page_location_kwargs = {}
         self.action = action
+        if self.destination is not None and self.action is not None:
+            raise ValueError("Only one of destination and action may be set")
         self.obj = obj
-        for k, v in kwargs.items():
-            if k in ALL_PAGE_LOCATION_KWARGS:
-                self.page_location_kwargs[k] = v
-            else:
-                raise ValueError("Invalid keyword argument {0}".format(k))
+        kwargs = dict(left=left, top=top, right=right, bottom=bottom, zoom=zoom)
+        self.page_location_kwargs = {k: v for k, v in kwargs.items() if v is not None}
         self.is_closed = False
         self.children: Iterable[OutlineItem] = []
 
@@ -128,13 +171,21 @@ class OutlineItem:
         else:
             oc_indicator = '[ ]'
         if self.destination is not None:
-            dest = self.destination
+            if isinstance(self.destination, Array):
+                # 12.3.2.2 Explicit destination
+                # [raw_page, /PageLocation.SomeThing, integer parameters for viewport]
+                raw_page = self.destination[0]
+                page = Page(raw_page)
+                dest = page.label
+            elif isinstance(self.destination, String):
+                # 12.3.2.2 Named destination
+                dest = f'<Named Destination: {self.destination}>'
         else:
             dest = '<Action>'
-        return '{1} {0.title} -> {2}'.format(self, oc_indicator, dest)
+        return f'{oc_indicator} {self.title} -> {dest}'
 
     def __repr__(self):
-        return '<{0.__class__.__name__}: "{0.title}">'.format(self)
+        return f'<pikepdf.{self.__class__.__name__}: "{self.title}">'
 
     @classmethod
     def from_dictionary_object(cls, obj: Dictionary):
@@ -146,7 +197,15 @@ class OutlineItem:
         """
         title = str(obj.Title)
         destination = obj.get(Name.Dest)
+        if destination is not None and not isinstance(destination, (Array, String)):
+            raise OutlineStructureError(
+                f"Unexpected object type in Outline's /Dest: {destination!r}"
+            )
         action = obj.get(Name.A)
+        if action is not None and not isinstance(action, Dictionary):
+            raise OutlineStructureError(
+                f"Unexpected object type in Outline's /A: {action!r}"
+            )
         return cls(title, destination=destination, action=action, obj=obj)
 
     def to_dictionary_object(self, pdf: Pdf, create_new: bool = False) -> Dictionary:
@@ -185,21 +244,23 @@ class OutlineItem:
 
 class Outline:
     """Maintains a intuitive interface for creating and editing PDF document outlines,
-    according to the PDF reference manual (ISO32000:2008) section 12.3.
+    according to the |pdfrm| section 12.3.
 
     Arguments:
         pdf: PDF document object.
         max_depth: Maximum recursion depth to consider when reading the outline.
         strict: If set to ``False`` (default) silently ignores structural errors.
-            Setting it to ``True`` raises a ``OutlineStructureError`` if any object
-            references re-occur while the outline is being read or written.
+            Setting it to ``True`` raises a
+            :class:`pikepdf.OutlineStructureError`
+            if any object references re-occur while the outline is being read or
+            written.
 
     See Also:
         :meth:`pikepdf.Pdf.open_outline`
     """
 
     def __init__(self, pdf: Pdf, max_depth: int = 15, strict: bool = False):
-        self._root = None
+        self._root: Optional[List[OutlineItem]] = None
         self._pdf = pdf
         self._max_depth = max_depth
         self._strict = strict
@@ -209,7 +270,7 @@ class Outline:
         return str(self.root)
 
     def __repr__(self):
-        return '<{0.__class__.__name__}: {1} items>'.format(self, len(self.root))
+        return f'<pikepdf.{self.__class__.__name__}: {len(self.root)} items>'
 
     def __enter__(self):
         self._updating = True
@@ -239,7 +300,7 @@ class Outline:
             if objgen in visited_objs:
                 if self._strict:
                     raise OutlineStructureError(
-                        "Outline object {0} reoccurred in structure".format(objgen)
+                        f"Outline object {objgen} reoccurred in structure"
                     )
                 out_obj = item.to_dictionary_object(self._pdf, create_new=True)
             else:
@@ -284,28 +345,34 @@ class Outline:
         level: int,
         visited_objs: Set[Tuple[int, int]],
     ):
-        current_obj = first_obj
+        current_obj: Optional[Dictionary] = first_obj
         while current_obj:
             objgen = current_obj.objgen
             if objgen in visited_objs:
                 if self._strict:
                     raise OutlineStructureError(
-                        "Outline object {0} reoccurred in structure".format(objgen)
+                        f"Outline object {objgen} reoccurred in structure"
                     )
                 return
             visited_objs.add(objgen)
 
             item = OutlineItem.from_dictionary_object(current_obj)
             first_child = current_obj.get(Name.First)
-            if first_child is not None and level < self._max_depth:
+            if isinstance(first_child, Dictionary) and level < self._max_depth:
                 self._load_level_outline(
                     first_child, item.children, level + 1, visited_objs
                 )
                 count = current_obj.get(Name.Count)
-                if count and count < 0:
+                if isinstance(count, int) and count < 0:
                     item.is_closed = True
             outline_items.append(item)
-            current_obj = current_obj.get(Name.Next)
+            next_obj = current_obj.get(Name.Next)
+            if next_obj is None or isinstance(next_obj, Dictionary):
+                current_obj = next_obj
+            else:
+                raise OutlineStructureError(
+                    f"Outline object {objgen} points to non-dictionary"
+                )
 
     def _save(self):
         if self._root is None:
@@ -328,7 +395,7 @@ class Outline:
             self._load_level_outline(first_obj, root, 0, set())
 
     @property
-    def root(self):
+    def root(self) -> Optional[List[OutlineItem]]:
         if self._root is None:
             self._load()
         return self._root
