@@ -23,6 +23,7 @@
 #include <qpdf/QPDFPageDocumentHelper.hh>
 #include <qpdf/Pl_Discard.hh>
 #include <qpdf/QPDFAcroFormDocumentHelper.hh>
+#include <qpdf/QPDFEmbeddedFileDocumentHelper.hh>
 
 #include <pybind11/stl.h>
 #include <pybind11/iostream.h>
@@ -125,9 +126,10 @@ std::shared_ptr<QPDF> open_pdf(py::object filename_or_stream,
     }
 
     if (!success) {
-        // LCOV_EXCL_LINE
+        // LCOV_EXCL_START
         throw std::logic_error(
             "open_pdf: should have succeeded or thrown a Python exception");
+        // LCOV_EXCL_STOP
     }
 
     if (inherit_page_attributes) {
@@ -183,8 +185,7 @@ void setup_encryption(QPDFWriter &w, py::object encryption)
         throw py::value_error("Invalid encryption level: must be 2, 3, 4 or 6");
 
     if (encryption_level == 5) {
-        auto warn = py::module_::import("warnings").attr("warn");
-        warn("Encryption R=5 is deprecated");
+        python_warning("Encryption R=5 is deprecated");
     }
 
     if (encryption.contains("owner")) {
@@ -271,7 +272,6 @@ void setup_encryption(QPDFWriter &w, py::object encryption)
             print,
             metadata);
     } else if (encryption_level == 5) {
-        // TODO WARNING
         w.setR5EncryptionParameters(user.c_str(),
             owner.c_str(),
             allow["accessibility"],
@@ -502,8 +502,8 @@ void init_qpdf(py::module_ &m)
             "Create a new empty PDF from stratch.")
         .def_static("_open",
             open_pdf,
-            "",
             py::arg("filename_or_stream"),
+            py::kw_only(),
             py::arg("password")                = "",
             py::arg("hex_password")            = false,
             py::arg("ignore_xref_streams")     = false,
@@ -646,8 +646,8 @@ void init_qpdf(py::module_ &m)
             )~~~")
         .def("_save",
             save_pdf,
-            "",
             py::arg("filename"),
+            py::kw_only(),
             py::arg("static_id")            = false,
             py::arg("preserve_pdfa")        = true,
             py::arg("min_version")          = "",
@@ -699,7 +699,7 @@ void init_qpdf(py::module_ &m)
             to that page, such as images on the page, may still be present.
 
             Retun type:
-                pikepdf._ObjectList
+                pikepdf._qpdf._ObjectList
             )~~~",
             py::return_value_policy::reference_internal)
         .def("make_indirect",
@@ -770,6 +770,10 @@ void init_qpdf(py::module_ &m)
             py::return_value_policy::reference_internal,
             py::keep_alive<1, 2>(),
             py::arg("h"))
+        .def("copy_foreign",
+            [](QPDF &q, QPDFPageObjectHelper &poh) -> QPDFPageObjectHelper {
+                return QPDFPageObjectHelper(q.copyForeignObject(poh.getObjectHandle()));
+            })
         .def("_replace_object",
             [](QPDF &q, std::pair<int, int> objgen, QPDFObjectHandle &h) {
                 q.replaceObject(objgen.first, objgen.second, h);
@@ -781,16 +785,9 @@ void init_qpdf(py::module_ &m)
                 q.swapObjects(o1, o2);
             })
         .def(
-            "_process",
-            [](QPDF &q, std::string description, py::bytes data) {
-                std::string s = data;
-                q.processMemoryFile(description.c_str(), s.data(), s.size());
-            },
-            R"~~~(
-            Process a new in-memory PDF, replacing the existing PDF
-
-            Used to implement Pdf.close().
-            )~~~")
+            "_close",
+            [](QPDF &q) { q.closeInputSource(); },
+            "Used to implement Pdf.close().")
         .def("_decode_all_streams_and_discard",
             [](QPDF &q) {
                 QPDFWriter w(q);
@@ -927,5 +924,18 @@ void init_qpdf(py::module_ &m)
 
             .. versionadded:: 2.11
             )~~~",
-            py::arg("mode") = "all"); // class Pdf
+            py::arg("mode") = "all") // class Pdf
+        .def_property_readonly(
+            "attachments",
+            [](QPDF &q) { return QPDFEmbeddedFileDocumentHelper(q); },
+            R"~~~(
+            Returns a mapping that provides access to all files attached to this PDF.
+
+            PDF supports attaching (or embedding, if you prefer) any other type of file,
+            including other PDFs. This property provides read and write access to
+            these objects by filename.
+
+            Returns:
+                pikepdf._qpdf.Attachments
+            )~~~");
 }
