@@ -1,3 +1,5 @@
+from typing import Type
+
 import pytest
 
 from pikepdf import Array, Dictionary, Name, Page, Pdf, Rectangle
@@ -17,7 +19,7 @@ def fourpages(resources):
 
 @pytest.fixture
 def graph_page(graph):
-    return Page(graph.pages[0])
+    return graph.pages[0]
 
 
 def test_page_boxes(graph_page):
@@ -122,9 +124,9 @@ def test_failed_add_page_cleanup():
 
 
 def test_formx(graph, outpdf):
-    formx = Page(graph.pages[0]).as_form_xobject()
+    formx = graph.pages[0].as_form_xobject()
     graph.add_blank_page()
-    new_page = Page(graph.pages[-1])
+    new_page = graph.pages[-1]
     formx_placed_name = new_page.add_resource(formx, Name.XObject)
     cs = new_page.calc_form_xobject_placement(
         formx, formx_placed_name, Rectangle(0, 0, 200, 200)
@@ -137,10 +139,11 @@ def test_formx(graph, outpdf):
 def test_fourpages_to_4up(fourpages, graph, outpdf):
     pdf = Pdf.new()
     pdf.add_blank_page(page_size=(1000, 1000))
-    page = Page(pdf.pages[0])
+    page = pdf.pages[0]
 
     pdf.pages.extend(fourpages.pages)
 
+    # Keep explicit Page(pdf.pages[..]) here
     page.add_overlay(pdf.pages[1], Rectangle(0, 500, 500, 1000))
     page.add_overlay(Page(pdf.pages[2]), Rectangle(500, 500, 1000, 1000))
     page.add_overlay(Page(pdf.pages[3]).as_form_xobject(), Rectangle(0, 0, 500, 500))
@@ -154,3 +157,31 @@ def test_fourpages_to_4up(fourpages, graph, outpdf):
     del pdf.pages[1:]
 
     pdf.save(outpdf)
+
+
+def test_cant_hash_page(graph):
+    with pytest.raises(TypeError, match="mutable"):
+        hash(graph.pages[0])
+
+
+def test_contents_add(graph):
+    graph.pages[0].contents_add(b'q Q', prepend=True)
+
+    new_cs = graph.make_stream(b'q Q')
+    graph.pages[0].contents_add(new_cs, prepend=False)
+
+    graph.pages[0].contents_coalesce()
+    assert graph.pages[0].Contents.read_bytes().startswith(b'q Q')
+    assert graph.pages[0].Contents.read_bytes().endswith(b'q Q')
+
+
+def test_page_attrs(graph):
+    # Test __getattr__
+    assert isinstance(graph.pages[0].Resources, Dictionary)
+
+    del graph.pages[0].Resources
+    with pytest.raises(AttributeError, match="can't delete"):
+        del graph.pages[0].obj
+    del graph.pages[0]['/Contents']
+
+    assert Name.Contents not in graph.pages[0] and Name.Resources not in graph.pages[0]
