@@ -111,8 +111,9 @@ class TestPasswords:
 
 class TestPermissions:
     def test_some_permissions_missing(self, resources):
-        with Pdf.open(resources / 'graph-encrypted.pdf', 'owner') as pdf:
-            assert pdf.allow.print_highres == pdf.allow.modify_annotation == False
+        with Pdf.open(resources / 'graph-encrypted.pdf', password='owner') as pdf:
+            assert not pdf.allow.print_highres
+            assert not pdf.allow.modify_annotation
             assert pdf.allow.print_lowres
 
     def test_permissions_all_true_not_encrypted(self, trivial):
@@ -143,9 +144,9 @@ class TestStreams:
         assert saved_file_contents == bio.read()
 
     def test_read_not_readable_file(self, outdir):
-        writable = (Path(outdir) / 'writeme.pdf').open('wb')
-        with pytest.raises(ValueError, match=r'not readable'):
-            Pdf.open(writable)
+        with (Path(outdir) / 'writeme.pdf').open('wb') as writable:
+            with pytest.raises(ValueError, match=r'not readable'):
+                Pdf.open(writable)
 
     def test_open_not_seekable_stream(self, resources):
         class UnseekableBytesIO(BytesIO):
@@ -160,10 +161,10 @@ class TestStreams:
 
 class TestMemory:
     def test_memory(self, resources):
-        pdf = (resources / 'pal-1bit-trivial.pdf').read_bytes()
+        data = (resources / 'pal-1bit-trivial.pdf').read_bytes()
         with pytest.warns(UserWarning, match="bytes-like object containing a PDF"):
             with pytest.raises(Exception):
-                pdf = Pdf.open(pdf)
+                Pdf.open(data)
 
 
 def test_remove_unreferenced(resources, outdir):
@@ -209,8 +210,9 @@ def test_min_and_force_version(trivial, outdir):
         assert pdf17.pdf_version == '1.7'
 
     pdf.save(outdir / '1.2.pdf', force_version='1.2')
-    pdf12 = Pdf.open(outdir / '1.2.pdf')
-    assert pdf12.pdf_version == '1.2'
+
+    with Pdf.open(outdir / '1.2.pdf') as pdf12:
+        assert pdf12.pdf_version == '1.2'
 
 
 def test_normalize_linearize(trivial, outdir):
@@ -297,6 +299,12 @@ def test_allow_overwriting_input_without_filename():
             pass
 
 
+def test_allow_overwriting_input_from_pdf_new():
+    pdf = Pdf.new()
+    with pytest.raises(ValueError, match="allow_overwriting_input=True"):
+        pdf.save()
+
+
 def test_check(resources):
     with pikepdf.open(resources / 'content-stream-errors.pdf') as pdf:
         problems = pdf.check()
@@ -334,12 +342,22 @@ def test_invalid_flate_compression_level():
         pikepdf._qpdf.set_flate_compression_level(99)
 
 
-if os.name == 'posix':
-    # Can't even let non-POSIX platforms see this marker e.g. by using .mark.skipif
-    # or they will fail on a pytest internalerror.
-    @pytest.mark.forked
-    def test_flate_compression_level():
-        pikepdf._qpdf.set_flate_compression_level(1)
+def test_flate_compression_level():
+    # While this function affects global state, we can test it safely because
+    # setting the value to -1 restores the default.
+    try:
+        pikepdf._qpdf.set_flate_compression_level(0)
+        pikepdf._qpdf.set_flate_compression_level(9)
+    finally:
+        pikepdf._qpdf.set_flate_compression_level(-1)
+
+
+def test_set_access_default_mmap():
+    initial = pikepdf._qpdf.get_access_default_mmap()
+    try:
+        pikepdf._qpdf.set_access_default_mmap(True)
+    finally:
+        pikepdf._qpdf.set_access_default_mmap(initial)
 
 
 def test_generate_appearance_streams(pdf_form):
